# coding: utf-8
from threading import Thread
import schedule, time
from dophon_db import properties
from dophon_logger import *

logger = get_logger(eval(properties.log_types))

_sched = schedule.Scheduler()

"""
调度器
author:CallMeE
date:2018-06-01
"""

logger.inject_logger(globals())


class sech_obj:
    def __init__(self, fun, delay):
        self.__fun = fun
        self.__delay = delay

    def enter(self):
        global _scheds
        _sched.every(self.__delay).seconds.do(self.__fun)

    def run_target(self):
        return self.__fun


def run():
    while True:
        _sched.run_pending()
        time.sleep(1)


def start_sech():
    """
    启动xml更新
    :return:
    """
    logger.info('xml自动更新调度启动')
    Thread(target=run).start()



# coding: utf-8
import re, uuid, zlib
from dophon_db.const import *

"""
binlog工具
author:CallMeE
date:2018-06-01
"""


def zip_as_bin(file):
    f = open(file, 'r', encoding='utf8')
    bstr = f.read()
    # 去除注释与空格,换行等
    bstr = re.sub(REGIXSTR.get_at_least_one_blank, ' ', re.sub('<!--.*-->', ' ', bstr))
    # 压缩内容
    z_s = zip_str(bstr)
    bs = []
    for b in str.encode(bstr):
        bs.append(b)
    # 返回唯一标识值(文件存储用文件名原子的uuid,文件标识用压缩内容原子的uuid)
    return uuid.uuid3(uuid.NAMESPACE_DNS, bstr)


# 暂时弃用
def bindigits(n, bits):
    s = bin(n & int("1" * bits, 2))[2:]
    return ("{0:0>%s}" % (bits)).format(s)


def zip_str(string):
    # 压缩内容
    z_result = zlib.compress(string.encode(encoding='utf8'))
    return z_result


def un_zip_str(string):
    # 解压内容
    un_z_result = zlib.decompress(string).decode(encoding='utf8')
    return un_z_result



import re

from .remote.Cell import Cell
from dophon_logger import *

from dophon_db import properties
from dophon_db.const import *
from . import PageHelper
from .core import CurObj

logger = get_logger(eval(properties.log_types))

obj_manager = {}

# 定义项目路径
project_path = properties.project_root


class PageObj(CurObj):
    def pageable_exe_sql(self, methodName: str = '', pageInfo: dict = None, args=()) -> dict:
        result_list = self.exe_sql(methodName=methodName, pageInfo=pageInfo, args=args)
        if pageInfo:
            # 获取无分页语句
            un_page_sql = self.get_sql(methodName=methodName, pageInfo=None, args=args)
            if re.match(REGIXSTR.select_sql_check, un_page_sql):
                # sql语句判断(非查询不作分页信息处理)
                un_page_sql = re.sub('^\\s*(s|S)(e|E)(l|L)(e|E)(c|C)(t|T)\\s+.+(f|F)(r|R)(o|O)(m|M)',
                                     'SELECT COUNT(*) FROM',
                                     un_page_sql)
                conn = self._pool.getConn()
                connect = conn.getConnect()
                cursor = connect.cursor()
                cursor.execute(un_page_sql)
                count_items = cursor.fetchall()[0][0]
                connect.commit()
                self._pool.closeConn(conn)
                result = PageHelper.fix_page_info(pageInfo)
                import math
                result['total_page'] = math.ceil(count_items / result['page_size'])
                result['list'] = result_list
                return result
        return result_list


def get_db_obj(path, debug: bool = False, auto_fix: bool = False, proxy_getter: bool = True):
    """
    获取数据表实例
    :param path: xml文件路径
    :param debug: 是否开启调试模式
    :param auto_fix: 是否开启路径修复模式(损耗资源) <===  待测试
    :return: xml对应实例
    """
    # if not pool:
    #     logger.error('连接池未定义')
    #     raise Exception('连接池未定义')
    # if 0 >= pool.size():
    #     配置属性生命周期过短,拟用__import__导入减轻内存废址
    # prop = __import__('properties')
    # if hasattr(prop, 'pool_conn_num'):
    #     pool.initPool(getattr(prop, 'pool_conn_num'), Connection.Connection)
    # else:
    #     初始5个连接
    # pool.initPool(5, Connection.Connection)
    if isinstance(path, str):
        if auto_fix:
            pattern = re.sub(r'\\', r'\\\\', re.sub('/', '\\/', project_path))
            if not re.search(pattern, path):
                r_path = str(project_path) + str(path)
            else:
                r_path = str(path)
        else:
            r_path = path
    elif isinstance(path, Cell):
        r_path = path.getPath()
    else:
        raise TypeError('结果集路径类型错误')
    # 数据语句对象改为单例模式获取
    if r_path in obj_manager:
        return obj_manager[r_path]
    singleton_obj = CurObj(r_path, True, debug, proxy_getter=proxy_getter)
    obj_manager[r_path] = singleton_obj
    return singleton_obj


def get_pg_obj(path, debug: bool = False, auto_fix: bool = False, proxy_getter: bool = True):
    """
    获取数据表实例
    :param path: xml文件路径
    :param debug: 是否开启调试模式
    :param auto_fix: 是否开启路径修复模式(损耗资源) <===  待测试
    :return: xml对应实例
    """
    # if pool is None:
    #     logger.error('连接池未定义')
    #     raise Exception('连接池未定义')
    # if 0 >= pool.size():
    # 配置属性生命周期过短,拟用__import__导入减轻内存废址
    # prop = __import__('properties')
    # if hasattr(prop, 'pool_conn_num'):
    #     pool.initPool(getattr(prop, 'pool_conn_num'), Connection.Connection)
    # else:
    # 初始5个连接
    # pool.initPool(5, Connection.Connection)
    if isinstance(path, str):
        if auto_fix:
            pattern = re.sub(r'\\', r'\\\\', re.sub('/', '\\/', project_path))
            if not re.search(pattern, path):
                r_path = str(project_path) + str(path)
            else:
                r_path = str(path)
        else:
            r_path = path
    elif isinstance(path, Cell):
        r_path = path.getPath()
    else:
        raise TypeError('结果集路径类型错误')
    # 数据语句对象改为单例模式获取
    if r_path in obj_manager:
        return obj_manager[r_path]
    singleton_obj = PageObj(r_path, True, debug, proxy_getter=proxy_getter)
    obj_manager[r_path] = singleton_obj
    return singleton_obj


getDbObj = get_db_obj
getPgObj = get_pg_obj



import types

import re

import os
from pymysql import escape_dict, escape_sequence, escape_string

escape_charset = 'utf-8'


def escape(val):
    """
    转义值,防sql注入
    :param val:
    :return:
    """
    if isinstance(val, dict):
        return escape_dict(val, charset=escape_charset)
    elif isinstance(val, (list, set,)):
        return escape_sequence(val, charset=escape_charset)
    else:
        return escape_string(val)


class CastableObject:
    """
    转换对象公共类
    """

    @classmethod
    def parse(clz, exe_result):
        return into_obj(exe_result=exe_result, clz=clz)


def create_cast_obj(kwargs: dict, clz_name: str):
    """
    创造类
    :param kwargs:
    :param clz_name:
    :return:
    """
    class_obj = type(clz_name, (CastableObject,),
                     {})
    for k, v in kwargs.items():
        setter_code = compile(
            'def setter_' + k + '(self,value):' +
            '\n\tself._' + k + ' = value',
            '',
            'exec'
        )
        setter_function_code = [c for c in setter_code.co_consts if isinstance(c, types.CodeType)][0]
        setter_method = types.FunctionType(setter_function_code, {})

        getter_code = compile(
            'def getter_' + k + '(self):' +
            '\n\treturn self._' + k,
            '',
            'exec'
        )
        getter_function_code = [c for c in getter_code.co_consts if isinstance(c, types.CodeType)][0]
        getter_method = types.FunctionType(getter_function_code, {})

        setattr(
            class_obj,
            '_' + k,
            v,
        )

        setattr(
            class_obj,
            k,
            property(getter_method, setter_method)
        )
    return class_obj()


def into_obj(exe_result, clz: CastableObject):
    """
    结果集转换对象
    :param exe_result: 结果集<list,dict>
    :param clz: 转换对象类
    :return:
    """
    if isinstance(exe_result, (list,)):
        return [into_obj_single(item, clz) for item in exe_result]
    elif isinstance(exe_result, (dict,)):
        return into_obj_single(exe_result, clz)
    else:
        raise Exception('实例转换异常')


def into_obj_single(exe_result_item: dict, clz: CastableObject):
    """
    结果集转换对象:单个
    :param exe_result_item: 结果集<dict>
    :param clz: 转换对象类
    :return:
    """
    result_obj = create_cast_obj(exe_result_item, getattr(clz, '__name__'))
    return result_obj


def transfer_to_mapper_key(path: str):
    return re.sub('(\\..*)|:', '', '_'.join(path.split(os.sep)))



import math
from dophon_db import utils, properties

from dophon_logger import *

logger = get_logger(eval(properties.log_types))

"""
查询语句结构映射

结构:
select
<Field>
from
<tables>
where
<param_dicts>
<sort_param>
"""

logger.inject_logger(globals())


def get_connection(target):
    """
    从连接池中获取连接
    :return:
    """
    return target.pool_getter().get_conn()


class Selelct:
    """
    查询结构类
    """

    def before_select(self, fields_list: list, has_where: bool) -> str:
        result = f"""SELECT {(getattr(self, 'fields')(fields_list) if fields_list else ' * ')} FROM """f"""{(
            getattr(self, 'table_map_key') + ((' AS ' + getattr(self, '__alias'))
                                              if getattr(self, '__alias') != getattr(self, 'table_map_key') else '')
            if not hasattr(self, '__join_list') else (getattr(self, 'exe_join')()))}{(
            getattr(self, 'where')() if has_where else '')} {getattr(self, '__execuable_limit_sql')
        if hasattr(self, '__execuable_limit_sql') and hasattr(self, '__limit_start') else ''}"""
        return result.replace('\n', ' ')

    def select(self, fields: list = [], has_where: bool = True) -> list:
        """
        查询并获取该表结果集
        :param fields: 列参
        :return: <list> 多条结果列表
        """
        sql = self.before_select(fields, has_where)
        logger.debug('执行: %s', sql)
        result = []
        connection = get_connection(self)
        cursor = connection.cursor()
        cursor.execute(sql)
        if not sql.startswith('select') and not sql.startswith('SELECT'):
            data = [[cursor.rowcount]]
            description = [['row_count']]
        else:
            data = cursor.fetchall()
            description = cursor.description
        connection.commit()
        result = utils.sort_result(data, description, result)
        # 清除当前分页信息
        if hasattr(self, '__execuable_limit_sql') and hasattr(self, '__limit_start'):
            self.clear_limit_info()
        return self.page_filter(result) if getattr(self, '__page_switch', False) else result

    def select_one(self, fields: list = []) -> dict:
        """
        查询一条结果集
        :param fields: 列参
        :return: <dict> 单条结果集字典
        """
        if hasattr(self, '__field_callable_list') and len(getattr(self, '__field_callable_list')) > 0:
            pass
        else:
            logger.warn('无法预料的唯一结果集,找不到查询过滤条件')
            # raise Exception('无法预料的唯一结果集,找不到查询过滤条件')

        # 存在可生成的查询条件
        result = self.select(fields=fields)
        assert len(result) >= 0, '神奇的错误'
        assert len(result) <= 1, '过多结果集'
        return result.pop(0) if len(result) > 0 else {}

    def select_all(self, fields: list = [], ignore_fields_warning: bool = True) -> list:
        """
        同select
        :param fields:
        :return:
        """
        if not ignore_fields_warning and hasattr(self, '__field_callable_list') and len(
                getattr(self, '__field_callable_list')) > 0:
            logger.warning('警告:存在查询过滤条件 %s ', str(getattr(self, '__field_callable_list')))
        result = self.select(fields=fields, has_where=False)
        return self.page_filter(result) if getattr(self, '__page_switch', False) else result

    def before_count(self, field_name: str, has_where: bool) -> str:
        count_str = getattr(self, 'fields')([field_name]) if field_name else ' * '
        assert count_str, '无此列名'
        result = f"""SELECT COUNT({count_str}) AS count FROM """f"""{(
            getattr(self, 'table_map_key') + ((' AS ' + getattr(self, '__alias'))
                                              if getattr(self, '__alias') != getattr(self, 'table_map_key') else '')
            if not hasattr(self, '__join_list') else (getattr(self, 'exe_join')()))}{(
            getattr(self, 'where')() if has_where else '')} {getattr(self, '__execuable_limit_sql')
        if hasattr(self, '__execuable_limit_sql') and hasattr(self, '__limit_start') else ''}"""
        return result.replace('\n', ' ')

    def count(self, field: str = '', has_where: bool = True):
        """
        查询条数
        :return:
        """
        sql = self.before_count(field, has_where)
        logger.debug('执行: %s', sql)
        result = []
        connection = get_connection(self)
        cursor = connection.cursor()
        cursor.execute(sql)
        if not sql.startswith('select') and not sql.startswith('SELECT'):
            data = [[cursor.rowcount]]
            description = [['row_count']]
        else:
            data = cursor.fetchall()
            description = cursor.description
        connection.commit()
        result = utils.sort_result(data, description, result)[0]
        return result

    def page_filter(self, result: list) -> dict:
        """
        分页过滤器

        注意: pages值由于python3问题为一个可迭代对象
        :param result:
        :return:
        """
        c_num = int(self.count()['count'])
        page_size = getattr(self, '__page_size', -1)
        pages_size = getattr(self, '__pages_size')
        page_num = getattr(self, '__page_num', 0)
        assert page_size > -1, '不存在分页信息'
        total_page = math.ceil((c_num / page_size))
        # 清除分页信息
        self.clear_page_info()
        return {
            'list': result if isinstance(result, list) else [result],
            'total': c_num,
            'total_page': total_page,
            'pages': list(
                range(page_num, (total_page if total_page <= (page_num + pages_size) else (page_num + pages_size)))
            )
        }


class Insert:
    """
    新增结构类
    """

    def before_insert(self):
        result = 'INSERT INTO ' + \
                 getattr(self, 'table_map_key') + ' ' + \
                 getattr(self, 'values')()
        return result

    def insert(self) -> int:
        """
        新增结果集
        :return: <int> 影响行数
                [{'row_count': '0'}]
        """
        sql = self.before_insert()
        logger.debug('执行: %s', sql)
        result = []
        connection = get_connection(self)
        cursor = connection.cursor()
        try:
            cursor.execute(sql)
            if not sql.startswith('select') and not sql.startswith('SELECT'):
                data = [[cursor.rowcount]]
                description = [['row_count']]
            else:
                data = cursor.fetchall()
                description = cursor.description
            connection.commit()
            result = utils.sort_result(data, description, result)[0]['row_count']
            return int(result)
        except Exception as e:
            logger.error('%s', e)
            connection.rollback()
            return 0


class Update():
    """
    更新结构类
    """

    def before_update(self, update: list, where: list):
        result = 'UPDATE ' + getattr(self, 'table_map_key') + \
                 (
                     (' AS ' + getattr(self, '__alias'))
                     if getattr(self, '__alias') != getattr(self, 'table_map_key') else ''
                 ) + \
                 getattr(self, 'set')(update) + \
                 getattr(self, 'where')(where)
        return result

    def update(self, update: list = [], where: list = []) -> int:
        """
        更新结果集
        :param update: 更新列参
        :param where: 条件列参
        :return: <int> 影响行数
        """
        sql = self.before_update(update, where)
        logger.debug('执行: %s', sql)
        result = []
        connection = get_connection(self)
        cursor = connection.cursor()
        try:
            cursor.execute(sql)
            if not sql.startswith('select') and not sql.startswith('SELECT'):
                data = [[cursor.rowcount]]
                description = [['row_count']]
            else:
                data = cursor.fetchall()
                description = cursor.description
            connection.commit()
            result = utils.sort_result(data, description, result)[0]['row_count']
            return int(result)
        except Exception as e:
            logger.error('%s', e)
            connection.rollback()
            return 0


class Delete():
    """
    删除结构类
    """

    def before_delete(self, where: list):
        result = 'DELETE FROM ' + getattr(self, 'table_map_key') + \
                 ' ' + getattr(self, 'where')(where, be_alias=False)
        return result

    def delete(self, where: list = []) -> int:
        """
        删除结果集

        :param where: 条件列参
        :return: <int> 影响行数
        """
        sql = self.before_delete(where)
        logger.debug('执行: %s', sql)
        result = []
        connection = get_connection(self)
        cursor = connection.cursor()
        try:
            cursor.execute(sql)
            if not sql.startswith('select') and not sql.startswith('SELECT'):
                data = [[cursor.rowcount]]
                description = [['row_count']]
            else:
                data = cursor.fetchall()
                description = cursor.description
            connection.commit()
            result = utils.sort_result(data, description, result)[0]['row_count']
            return int(result)
        except Exception as e:
            logger.error('%s', e)
            connection.rollback()
            return 0


class Struct(Selelct, Insert, Update, Delete):
    """
    查询语句结构类
    """



# coding: utf-8
import os
import stat
import time
import uuid
from urllib import request

from dophon_logger import *
from dophon_db import properties
from dophon_db import mysql

logger = get_logger(eval(properties.log_types))

"""
远程xml映射实例模板
author:CallMeE
date:2018-06-01


# 开启远程xml需要导入以下模块
# import mysql.remote as remote

demo:
remote_cell = remote.get_cell('ShopGoodsMapper.xml', remote_path='http://127.0.0.1:8400/member/export/xml/ShopGoodsMapper.xml')
obj1 = getDbObj(remote_cell.getPath(), debug=True)
或者
obj1 = getDbObj(remote_cell, debug=True)

注意！！！
read_only设置远程xml文件是否为只读，注意防止与自动增量更新冲突
"""


logger.inject_logger(globals())


class Cell():
    def __init__(self, file_name='', remote_path='', read_only=False):
        self._file_name = file_name
        self._remote_path = remote_path
        self._uid = uuid.uuid5(uuid.NAMESPACE_DNS, file_name)
        # 创建临时文件夹(安全性考虑，采用多层目录)
        self._file_path = mysql.project_path + sort_path(self._uid)
        # 检查路径
        if os.path.exists(self._file_path):
            pass
        else:
            os.makedirs(self._file_path)
        while True:
            # 下载远程文件
            try:
                response = request.urlretrieve(url=remote_path, filename=self._file_path + '/' + self._file_name)
                logger.debug('加载远程mapper：' + response[0])
                # 放置路径
                self._abs_path = response[0]
                if read_only:
                    self.lock_to_read()
                break
            except Exception as e:
                logger.error(e)
                logger.error('连接远程计算机失败,请检查连接,3秒后重试(' + str(id(self)) + ')')
                time.sleep(3)

    # 重新加载文件
    def reload_file(self):
        # 放开写入权限
        if self.is_only_read():
            self.unlock_to_read()
            response = request.urlretrieve(url=self._remote_path, filename=self._file_path + '/' + self._file_name)
            self.lock_to_read()
        else:
            response = request.urlretrieve(url=self._remote_path, filename=self._file_path + '/' + self._file_name)
        logger.debug('加载远程mapper：' + response[0])
        # 放置路径
        self._abs_path = response[0]
        # 链式调用（非必需）
        return self

    # 进入调度定时更新文件
    def reload_file_round(self, minute):
        Schued.sech_obj(self.reload_file, minute * 60).enter()
        # 链式调用（非必需）
        return self

    # 获取下载文件路径
    def getPath(self):
        return self._abs_path

    # 另一种方式获取路径
    def __str__(self):
        return self.getPath()

    # 锁定文件为只读
    def lock_to_read(self):
        os.chmod(self._abs_path, stat.S_IREAD)

    # 解除文件只读
    def unlock_to_read(self):
        os.chmod(self._abs_path, stat.S_IWRITE)

    # 判断文件是否只读
    def is_only_read(self):
        try:
            with open(self._abs_path, "r+") as fr:
                return False
        except IOError as e:
            if "[Errno 13] Permission denied" in str(e):
                return True
            else:
                logger.error(str(e))
                return False


def sort_path(path_str):
    result = '/.mapper'
    for s in str(path_str):
        result = result + '/' + s
    return result


# 工厂模式获取实例
# def get_cell(file_name, remote_path, read_only):
#     return Cell(file_name=file_name, remote_path=remote_path, read_only=read_only)



from dophon_logger import *

from dophon_db import properties
from . import PageHelper
from ..sql_util import *
from . import cluster
from .core import *

logger = get_logger(eval(properties.log_types))

ALL = -1
SINGLE = 0
CLUSTER = 1

# 默认优先模式是分片
priority_mode = properties.db_mode if isinstance(properties.db_mode, int) else eval(properties.db_mode)

db_cluster = cluster.get_db

from .single import getDbObj, getPgObj

db_obj = None
pg_obj = None


def assert_single():
    return (db_obj if priority_mode == SINGLE or priority_mode == ALL else True) \
           and \
           (pg_obj if priority_mode == SINGLE or priority_mode == ALL else True)


def assert_cluster():
    return db_cluster if priority_mode == CLUSTER or priority_mode == ALL else True


def mode_assert(mode_type: str):
    def fun(f):
        def arg_bucket(*args, **kwargs):
            assert priority_mode == eval(mode_type) or priority_mode == ALL, f'模式异常,请切换至({mode_type})或(ALL)'
            return f(*args, **kwargs)

        return arg_bucket

    return fun


if priority_mode == CLUSTER:
    cluster.init_cluster_manager()
    # assert assert_cluster(), f'未切换到使用多源模式,当前模式为(SINGLE)'
if priority_mode == SINGLE:
    db_obj = mode_assert('SINGLE')(getDbObj)
    pg_obj = mode_assert('SINGLE')(getPgObj)
    # assert assert_single(), f'未切换到使用单源模式,当前模式为(CLUSTER)'
if priority_mode == ALL:
    cluster.init_cluster_manager()
    db_obj = mode_assert('SINGLE')(getDbObj)
    pg_obj = mode_assert('SINGLE')(getPgObj)
    # assert assert_cluster(), f'未切换到使用多源模式,当前模式为(SINGLE)'
    # assert assert_single(), f'未切换到使用单源模式,当前模式为(CLUSTER)'

__all__ = ['db_obj', 'pg_obj', 'db_cluster']



import re
import time
from threading import Thread

from dophon_logger import *

from dophon_db import properties
from dophon_db.mysql.orm import db_obj
from dophon_db.utils import *

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())

manager_map = {}

singleton_dbm = None


class ManagerObject:
    pass


def manager_map_to_object():
    result = ManagerObject()
    for k, v in manager_map.items():
        setattr(result, k, v)
    return result


def init_orm_manager(table_list: list = [], conn_kwargs: dict = {}, orm_pre_load: bool = properties.orm_pre_load):
    global manager_map, singleton_dbm
    manager = db_obj.OrmManager()
    # print(conn_kwargs)
    db_obj.init_pool_in_manager(manager, conn_kwargs)
    if table_list or orm_pre_load:
        db_obj.init_tables_in_db(manager, table_list, conn_kwargs)
    else:
        pass
        # logger.info('对象映射改为运行时映射')
    if conn_kwargs:
        manager_label = conn_kwargs['alias']
        manager_map[manager_label] = manager
        return manager_map_to_object()
    else:
        if singleton_dbm:
            return singleton_dbm
        singleton_dbm = manager
        # logger.info(manager_map)
        return manager


def init_cluster_orm_manager():
    global manager_map
    if manager_map:
        return manager_map
    db_cluster_info = get_db_cluster_info()
    for cluster_info in db_cluster_info:
        for cluster_name in cluster_info:
            logger.debug('初始化分片%s数据库 : %s' % (cluster_name, '::'.join(
                [(str(re.sub('_', '', k)) + '<' + str(v) + '>') for
                 k, v in cluster_info[cluster_name].items() if not re.match('.*(user|password).*', k)]
            ),))
            # orm内部管理器初始化方式启动
            # Thread(target=db_obj.init_tables_in_db, args=(manager,),
            #        kwargs={'conn_kwargs': cluster_info[cluster_name]}).start()
            Thread(target=init_orm_manager,
                   kwargs={
                       'table_list': cluster_info[cluster_name]['table_list'],
                       'conn_kwargs': cluster_info[cluster_name]
                   }).start()
    while len(manager_map) != len(db_cluster_info):
        # print(len(manager_map),'---',len(db_cluster_info))
        time.sleep(1)
    return manager_map


class ClusterManager:
    __map = {}

    def __init__(self):
        self.__map = init_cluster_orm_manager()

    def __getattr__(self, name):
        logger.debug('获取相应对象管理')
        result = None
        print(self.__map)
        for k, v in self.__map.items():
            # print(k,'---',v)
            if not result:
                if v.has_table(name):
                    result = eval('v.' + name)
                else:
                    try:
                        result = eval('v.' + name)
                    except KeyError as ke:
                        # print(ke)
                        pass
        if result:
            return result
        else:
            raise Exception('无法识别的表名')



from dophon_logger import *

from dophon_db import properties
from dophon_db.mysql.orm.manager_init import *

logger = get_logger(eval(properties.log_types))

logger.inject_logger(globals())


def init_orm(table_list=[], conn_kwargs={}):
    """
    初始化orm
    :return:
    """
    if getattr(properties, 'db_cluster', []):
        logger.info('分片数据库初始化')
        return ClusterManager()
    else:
        return init_orm_manager(table_list, conn_kwargs)



# coding: utf-8
#  连接
import datetime
import threading

from dophon_logger import *

from dophon_db import properties
from dophon_db.mysql import Connection

logger = get_logger(eval(properties.log_types))

"""
连接池
author:CallMeE
date:2018-06-01
"""

logger.inject_logger(globals())

single_pool = None  # 单例模式获取连接池
cluster_pool = None  # 单例模式获取连接池

pool_action_log = {}

lock = threading.Lock()  # 全局线程锁


def record_support(f):
    def inner_method(*args, **kwargs):
        result = f(*args, **kwargs)
        obj_id = id(result)
        pool_action_log[obj_id] = datetime.datetime.now().timestamp()
        return result

    return inner_method


def record_return(f):
    def inner_method(*args, **kwargs):
        obj_id = id(args[1])
        if hasattr(properties, 'db_pool_exe_time') and getattr(properties, 'db_pool_exe_time'):
            logger.debug(
                '用时:' + (datetime.datetime.now().timestamp() - pool_action_log[obj_id]) + '毫秒'
            )
        f(*args, **kwargs)

    return inner_method


def get_pool(conn_kwargs: dict = {}):
    return get_single_pool(conn_kwargs)


def get_single_pool(conn_kwargs: dict = {}):
    global single_pool
    if single_pool:
        return single_pool
    pool = Pool()
    pool.init_pool(properties.pool_conn_num, Connection.Connection, conn_kwargs=conn_kwargs)
    single_pool = pool
    return single_pool


class Pool():
    _size = 0
    cache_conn_kwargs = {}

    # 初始化连接池
    def init_pool(self, num: int, Conn: Connection, conn_kwargs: dict = {}):
        _pool = []
        self._Conn = Conn
        self.cache_conn_kwargs = conn_kwargs
        for item_c in range(num):
            # 遍历定义连接放入连接池
            conn = Conn(**conn_kwargs)
            _pool.append(conn)
        self._pool = _pool
        self._size = num
        return self

    def __init__(self):
        logger.debug(f'初始化连接池 => ({id(self)})')

    # 定义取出连接
    @record_support
    def get_conn(self) -> Connection:
        __pool = self._pool
        if __pool:
            lock.acquire(blocking=True)
            currConn = __pool.pop(0)
            if currConn.test_conn():
                # 连接有效
                # 不作处理
                pass
            else:
                logger.error('连接无效')
                currConn.re_conn()
            lock.release()
            return currConn
        else:
            # 连接数不足则新增连接
            conn = Connection.Connection(**self.cache_conn_kwargs)
            self._pool.append(conn)
            return self.get_conn()

    # 定义归还连接
    @record_return
    def close_conn(self, conn):
        self._pool.append(conn)

    # 定义查询连接池连接数
    def size(self):
        return self._size

    # 定义释放所有连接
    def free_pool(self):
        for conn in self._pool:
            conn.getConnect().close()
